import com.ibm.ruler.*;
import java.util.*;
import java.awt.Point;
//import java.lang.*;
/**
 * This is the class that you must implement to enable your ruler within
 * the CodeRuler environment. Adding code to these methods will give your ruler
 * its personality and allow it to compete.
 */
public class MyRuler extends Ruler {
	/* (non-Javadoc)
	 * @see com.ibm.ruler.Ruler#getRulerName()
	 */
	public static MyRuler me;
	Hashtable	knights;
	Hashtable	peasants;
	Hashtable	castles;

	IKnight	k[];
	ICastle	c[];
	IPeasant	p[];
	public int dir_arr[];
	public boolean bool_arr[];
	public Rect regions[];
	public int peas_cnt[];
	public Vector nearby_regions;
	public Vector reg_peasant_limits;
	public Random rand;
	Point	kpoints[];
	boolean	k_asgn_points[];
	int last_node_cnt;
	public int kcount;
	public int cast_kcount;
	public LinkedList work_queue, avail_queue;
	public int prod_cnt;

	class CastleQueue
	{
		public Castle castle;
		public int unit;
	}

	class Rect
	{
		public int minX, maxX, minY, maxY;

		public Rect(int mX, int mY, int xX, int xY)
		{
			minX = mX;
			minY = mY;
			maxX = xX;
			maxY = xY;
		}

		public Rect(double mX, double mY, double xX, double xY)
		{
			minX = (int)mX;
			minY = (int)mY;
			maxX = (int)xX;
			maxY = (int)xY;
		}

		public boolean inRect(int x, int y)
		{
			return (x >= minX && x <= maxX && y >= minY && y <= maxY);
		}

		public boolean outN(int y)
		{
			return (y < minY);
		}
		public boolean outS(int y)
		{
			return (y > maxY);
		}
		public boolean outE(int x)
		{
			return (x > maxX);
		}
		public boolean outW(int x)
		{
			return (x < minX);
		}

		public int distToSq(IObject o)
		{
			int x = minX + ((maxX - minX)/2);
			int y = minY + ((maxY - minY)/2);
			return (y-o.getY())*(y-o.getY()) + (x-o.getX())*(x-o.getX());
		}
	}

	int XCoord(int x, int dir)
	{
		switch(dir)
		{
			case MOVE_N:
			case MOVE_S:
				return x;
			case MOVE_NE:
			case MOVE_E:
			case MOVE_SE:
				return x + 1;
			case MOVE_NW:
			case MOVE_W:
			case MOVE_SW:
				return x - 1;
		}
		return x;
	}

	int YCoord(int y, int dir)
	{
		switch(dir)
		{
			case MOVE_W:
			case MOVE_E:
				return y;
			case MOVE_S:
			case MOVE_SE:
			case MOVE_SW:
				return y + 1;
			case MOVE_N:
			case MOVE_NE:
			case MOVE_NW:
				return y - 1;
		}
		return y;
	}

	Point Coord(int x, int y, int dir)
	{
		return new Point(XCoord(x, dir), YCoord(y, dir));
	}

	class Knight extends Object
	{
		public boolean visited;
		public IKnight obj;
		public int rand_cnt;
		public Point pt;
		public int pt_idx;
		public int cols_cnt;

		public Knight(IKnight k)
		{
			obj = k;
			rand_cnt = 0;
			pt = null;
			cols_cnt = 0;
			kcount++;

			ListIterator itr = work_queue.listIterator();
			while(itr.hasNext())
			{
				CastleQueue q = (CastleQueue)itr.next();
				if (q.unit == 2)
				{
					itr.remove();
					q.unit = 0;
					avail_queue.addLast(q);
					break;
				}
			}
		}

		public void move()
		{
			int i;
			for(i = 0; i < 8; i++)
			{
				int x = XCoord(obj.getX(), i);
				int y = YCoord(obj.getY(), i);
				if (x < 0 || x >= World.WIDTH)
					continue;
				if (y < 0 || y >= World.HEIGHT)
					continue;
				IObject o = World.getObjectAt(x, y);
				if (o != null && o.getRuler() != me && o.isAlive())
				{
					me.capture(obj, i);
					return;
				}
			}

			if (cols_cnt >= 5)
			{
				cols_cnt = 0;
				rand_cnt = 1;
			}

			if (rand_cnt < 5)
			{
				int dir = rand.nextInt(8) + 1;
				Point p = World.getPositionAfterMove(obj.getX(), obj.getY(), dir);
				if (p != null && (p.x != obj.getX() || p.y != obj.getY()) )
				{
					me.move(obj, dir);
					rand_cnt++;
				}
				return;
			}

			if (pt == null)
			{
				pt_idx = 0;
				for(i = 0; i < kpoints.length; i++)
				{
					if (kpoints[i] != null && !k_asgn_points[i])
					{
						pt_idx = i;
						pt = kpoints[pt_idx];
						k_asgn_points[pt_idx] = true;
						break;
					}
				}
			}

			if (pt != null)
			{
				int dir = MOVE_NONE;
				if(obj.getX() != pt.x || obj.getY() != pt.y)
				{
					dir = obj.getDirectionTo(pt.x, pt.y);
					Point p = World.getPositionAfterMove(obj.getX(), obj.getY(), dir);

					if (World.getObjectAt(p.x, p.y) != null)
						cols_cnt++;
					else
						cols_cnt = 0;

					me.move(obj, dir);
				}
				else
				{
					cols_cnt = 0;
					me.move(obj, MOVE_NONE);
				}
			}
			else
			{
				int dir = rand.nextInt(8)+1;
				Point p = World.getPositionAfterMove(obj.getX(), obj.getY(), dir);
				if (p != null && (p.x != obj.getX() || p.y != obj.getY()) )
					me.move(obj, dir);
			}
		}

		public void died()
		{
			if (pt != null)
			{
				k_asgn_points[pt_idx] = false;
				pt = null;
				kcount--;
			}
		}
	}

	class Peasant extends Object
	{
		public boolean visited;
		public IPeasant obj;
		public Rect grid;
		public int gridNum;
		public int stuck;

		public Peasant(IPeasant p)
		{
			int i;
			obj = p;
			stuck = 0;
			ICastle best = null;

			ICastle cas[] = getCastles();
			for(i = 0; i < cas.length; i++)
			{
				if (best == null ||
					(cas[i].getDistanceTo(obj.getX(), obj.getY()) <
						best.getDistanceTo(obj.getX(), obj.getY())) )
					best = cas[i];
			}
			cas = World.getOtherCastles();
			for(i = 0; i < cas.length; i++)
			{
				if (best == null ||
					(cas[i].getDistanceTo(obj.getX(), obj.getY()) <
						best.getDistanceTo(obj.getX(), obj.getY())) )
					best = cas[i];
			}
			int map[] = null;
			if (best != null)
				map = (int[])nearby_regions.elementAt(best.getId());

			if (map == null)
			{
				for(i = 1; i < peas_cnt.length; i++)
					if (peas_cnt[i] < peas_cnt[i-1])
						break;
				if (i == peas_cnt.length)
					i = 0;
				grid = regions[i];
				peas_cnt[i]++;
				gridNum = i;
			}
			else
			{
				int limit =((Integer) reg_peasant_limits.elementAt(best.getId())).intValue();
				for(i = 1; i < limit; i++)
					if (peas_cnt[map[i]] < peas_cnt[map[i-1]])
						break;
				if (i >= limit)
					i = limit++;
				grid = regions[map[i]];
				peas_cnt[map[i]]++;
				gridNum = map[i];
				reg_peasant_limits.setElementAt(new Integer(limit), best.getId());
				// System.out.println(map + " (" + best.getId() + ") dispatching to " + gridNum);
			}

			ListIterator itr = work_queue.listIterator();
			while(itr.hasNext())
			{
				CastleQueue q = (CastleQueue)itr.next();
				if (q.unit == 1)
				{
					work_queue.remove(q);
					q.unit = 0;
					avail_queue.addLast(q);
					break;
				}
			}
		}

		public void rand_move(int dirs[], int ndirs)
		{
			int i, j, a, b;
			boolean possible = false;
			boolean moved = false;
			for(i = 0; i < 8; i++)
				bool_arr[i] = false;

			for(j = 0; j < ndirs; j++)
			{
				for(int k = j + 1; k < ndirs; k++)
					if (rand.nextInt(2) == 1)
					{
						int t = dirs[j];
						dirs[j] = dirs[k];
						dirs[k] = t;
					}
			}

			for(i = 0; i < ndirs; i++)
			{
				int x = XCoord(obj.getX(), dirs[i]);
				int y = YCoord(obj.getY(), dirs[i]);
				if (x < 0 || x >= World.WIDTH)
					continue;
				if (y < 0 || y >= World.HEIGHT)
					continue;
				if (null != World.getObjectAt(x, y))
					continue;
				bool_arr[i] = possible = true;
				if (World.getLandOwner(x, y) != me)
				{
					me.move(obj, dirs[i]);
					moved = true;
					break;
				}
			}

			if (!moved && possible)
			{
				for(i = 0; i < 8; i++)
				{
					if (bool_arr[i])
					{
						me.move(obj, dirs[i]);
						break;
					}
				}
				moved = true;
			}

			if (moved)
				stuck = 0;
			else
				stuck++;
		}

		public void safe_squares()
		{
			int i, j;
			for(i = 0; i < 8; i++)
				bool_arr[i] = true;

			for(i = 0; i < 8; i++)
			{
				int x = XCoord(obj.getX(), i);
				int y = YCoord(obj.getY(), i);
				if (x < 0 || x >= World.WIDTH)
				{
					bool_arr[i] = false;
					continue;
				}
				if (y < 0 || y >= World.HEIGHT)
				{
					bool_arr[i] = false;
					continue;
				}

				for(j = 0; j < 8; j++)
				{
					int cx = XCoord(x, j);
					int cy = YCoord(y, j);
					if (cx < 0 || cx >= World.WIDTH)
						continue;
					if (cy < 0 || cy >= World.HEIGHT)
						continue;

					IObject o = World.getObjectAt(cx, cy);
					if (o == null)
						continue;
					if (o.getRuler() != me)
					{
						bool_arr[i] = false;
						break;
					}
				}
			}
		}

		public void move()
		{
			int i;
			int ndirs = 0;
			int x = obj.getX();
			int y = obj.getY();

			safe_squares();

			if (stuck > 10 || grid.inRect(x, y))
			{
				for(i = 0; i < 8; i++)
					if (bool_arr[i])
						dir_arr[ndirs++] = i + 1;
			}
			else
			{
				ndirs = 0;
				if (bool_arr[MOVE_W-1] && grid.outE(x))
					dir_arr[ndirs++] = MOVE_W;
				if (bool_arr[MOVE_E-1] && grid.outW(x))
					dir_arr[ndirs++] = MOVE_E;
				if (bool_arr[MOVE_N-1] && grid.outS(y))
					dir_arr[ndirs++] = MOVE_N;
				if (bool_arr[MOVE_S-1] && grid.outN(y))
					dir_arr[ndirs++] = MOVE_S;

				if (bool_arr[MOVE_SW-1] && grid.outN(y) && grid.outE(x))
					dir_arr[ndirs++] = MOVE_SW;
				if (bool_arr[MOVE_SE-1] && grid.outN(y) && grid.outW(x))
					dir_arr[ndirs++] = MOVE_SE;
				if (bool_arr[MOVE_NW-1] && grid.outS(y) && grid.outE(x))
					dir_arr[ndirs++] = MOVE_NW;
				if (bool_arr[MOVE_NE-1] && grid.outS(y) && grid.outW(x))
					dir_arr[ndirs++] = MOVE_NE;
			}


			if (ndirs == 0)
			{
				for(i = 0; i < 8; i++)
					if (bool_arr[i])
						dir_arr[ndirs++] = i + 1;
			}

			if (ndirs > 0)
				rand_move(dir_arr, ndirs);

		}

		public void died()
		{
			peas_cnt[gridNum]--;
		}
	}

	class Castle extends Object
	{
		public boolean visited;
		public ICastle obj;
		public CastleQueue mQ;

		public Castle(ICastle c)
		{
			obj = c;
			mQ = new CastleQueue();
			mQ.castle = this;
			mQ.unit = 0;
			avail_queue.addLast(mQ);
		}

		public void move()
		{
			if( k.length < 7 )
				me.createKnights(obj);
			if (p.length < 20)
				me.createPeasants(obj);
			else if (k.length < 10)
				me.createKnights(obj);
			else if (k.length < (c.length + World.getOtherCastles().length)*8 && (k.length < p.length * 3))
				me.createKnights(obj);
			else if (k.length < p.length)
				me.createKnights(obj);
			else
				me.createPeasants(obj);
				
			/*if (mQ.unit == 0)
			{
				boolean knight = false;
				avail_queue.remove(mQ);
				if (kcount <= cast_kcount)
				{
					if ( (prod_cnt % 3) == 0 || (prod_cnt % 3) == 2)
					{
						mQ.unit = 2;
						me.createKnights(obj);
						knight = true;
					}
					prod_cnt++;
				}
				if (!knight)
				{
					mQ.unit = 1;
					me.createPeasants(obj);
				}
				work_queue.addLast(mQ);
			}*/
		}

		public void died()
		{
		}
	}

	public String getRulerName() {
		return "Sir Sleep-a-lot";
	}

	/* (non-Javadoc)
	 * @see com.ibm.ruler.Ruler#getSchoolName()
	 */
	public String getSchoolName() {
		return "UCSD";
	}

	/* (non-Javadoc)
	 * @see com.ibm.ruler.Ruler#initialize()
	 */
	public void initialize() {
		knights = new Hashtable(3000);
		peasants = new Hashtable(3000);
		castles = new Hashtable(20);
		me = this;
		dir_arr = new int[8];
		bool_arr = new boolean[9];
		peas_cnt = new int[64];
		rand = new Random();
		last_node_cnt = -1;
		kcount = 0;
		cast_kcount = 8*World.getOtherCastles().length+8;
		work_queue = new LinkedList();
		avail_queue = new LinkedList();
		prod_cnt = 0;
		Point	delay_p[];
		int delay_p_offset = 0;

		xrand = rand.nextInt(23443857);
		yrand=rand.nextInt(23443857);
		delay_p = new Point[50];

		regions = new Rect[64];

		for(int i = 0; i < 8; i++)
			for(int j = 0; j < 8; j++)
			{
				int r = i*8 + j;
				peas_cnt[r] = 0;
				regions[r] = new Rect(
					((double)i)*(((double)World.WIDTH)/8.0),
					((double)j)*((double)World.HEIGHT/8.0),
					((double)(i+1))*(((double)World.WIDTH)/8.0) - 1,
					((double)(j+1))*((double)World.HEIGHT/8.0) - 1);

			}

		int nc = World.getOtherCastles().length + 1;
		nearby_regions = new Vector(20);
		nearby_regions.setSize(20);
		reg_peasant_limits = new Vector(20);
		reg_peasant_limits.setSize(20);
		for(int i = 0; i < nc; i++)
		{
			ICastle cast;
			if (i ==  (nc-1))
				cast = getCastles()[0];
			else
				cast = World.getOtherCastles()[i];

			int map[] = new int[regions.length];
			for(int j = 0; j < regions.length; j++)
				map[j] = j;
			for(int j = 0; j < regions.length; j++)
				for(int k = j + 1; k < regions.length; k++)
				{
					if (regions[map[k]].distToSq(cast) < regions[map[j]].distToSq(cast))
					{
						int t = map[j];
						map[j] = map[k];
						map[k] = t;
					}
				}

			nearby_regions.setElementAt(map, cast.getId());
			reg_peasant_limits.setElementAt(new Integer(0), cast.getId());
		}

		ICastle	cas[] = World.getOtherCastles();
		ICastle mcas = getCastles()[0];
		for(int i = 0; i < cas.length; i++)
			for(int j = i+1; j < cas.length; j++)
			{
				if (mcas.getDistanceTo(cas[j].getX(), cas[j].getY()) <
						mcas.getDistanceTo(cas[i].getX(), cas[i].getY()) )
				{
					ICastle t = cas[i];
					cas[i] = cas[j];
					cas[j] = t;
				}
			}
		
		kpoints = new Point[8*cas.length+8];
		k_asgn_points = new boolean[8*cas.length+8];
		int ind = 0;

		for(int i = 1; i <= 8; i++)
			kpoints[ind++] = Coord(mcas.getX(), mcas.getY(), i);
		for(int j = 0; j < cas.length; j++)
		{
			for(int i = 1; i <= 8; i++)
			{
				if (i != 2 && i != 7)
					kpoints[ind++] = Coord(cas[j].getX(), cas[j].getY(), i);
				else
					delay_p[delay_p_offset++] = Coord(cas[j].getX(), cas[j].getY(), i);
			}
		}
		for(int i = 0; i < (8*cas.length+8); i++)
			k_asgn_points[i] = false;

		while(delay_p_offset > 0)
			kpoints[ind++] = delay_p[--delay_p_offset];
	}

	/* (non-Javadoc)
	 * @see com.ibm.ruler.Ruler#orderSubjects(int)
	 */
	public void orderSubjects(int lastMoveTime) {
		// put implementation here
		int i;
		Enumeration e;
		int node_limit = -1;

		if (lastMoveTime > 0 && last_node_cnt > 0)
		{
			double node_time = ((double)lastMoveTime)/((double)last_node_cnt);
			node_limit = 1 + (int)(((double)800)/node_time);
			// System.out.println("Last node count: " + last_node_cnt + ", node limit: " + node_limit);
			node_limit = 95;
		}
		
		
		last_node_cnt = 0;

		// Tag all nodes as not visited
		e = knights.elements();
		while(e.hasMoreElements())
			((Knight)e.nextElement()).visited = false;
		e = castles.elements();
		while(e.hasMoreElements())
			((Castle)e.nextElement()).visited = false;
		e = peasants.elements();
		while(e.hasMoreElements())
			((Peasant)e.nextElement()).visited = false;

		// cache the arrays
		k = getKnights();
		c = getCastles();
		p = getPeasants();

		// Call each wrappers move function, or create a wrapper if none
		//   exists.
		for(i = 0; i < k.length; i++)
		{
			Knight kn = (Knight)knights.get(k[i]);
			if (kn == null)
			{
				kn = new Knight(k[i]);
				knights.put(k[i], kn);
			}
			kn.visited = true;
			kn.move();
			last_node_cnt++;
			node_limit--;
		}

		double target_pct = 2;
		if (p.length > node_limit)
		{
			target_pct = ((double)node_limit)/((double)p.length);
			// System.out.println("Only processing " + (100.0 * target_pct) + "% of peasants!");
		}

		// int start = World.getCurrentTurn() % p.length;
		// int proc_cnt = 0, tot_cnt = 0;

		/*i = -1;
		while(i != start)
		{
			if (i == -1)
				i = start;

			if (tot_cnt == 0 ||
				(((double)proc_cnt)/((double)tot_cnt)) < target_pct )
			{
				Peasant ps = (Peasant)peasants.get(p[i]);
				if (ps == null)
				{
					ps = new Peasant(p[i]);
					peasants.put(p[i], ps);
				}
				ps.visited = true;
				ps.move();
				last_node_cnt++;
				proc_cnt++;
			}

			tot_cnt++;
			if (++i >= p.length)
				i = 0;
		}*/
		move_peasants();

		for(i = 0; i < c.length; i++)
		{
			Castle cs = (Castle)castles.get(c[i]);
			if (cs == null)
			{
				cs = new Castle(c[i]);
				castles.put(c[i], cs);
			}
			cs.visited = true;
			cs.move();
		}

		// Prune dead objects
		e = knights.elements();
		while(e.hasMoreElements())
		{
			Knight kn = (Knight)e.nextElement();
			if (!kn.visited)
			{
				kn.died();
				knights.remove(kn.obj);
			}
		}

		/*e = peasants.elements();
		while(e.hasMoreElements())
		{
			Peasant ps = (Peasant)e.nextElement();
			if (!ps.visited)
			{
				ps.died();
				peasants.remove(ps.obj);
			}
		}*/

		e = castles.elements();
		while(e.hasMoreElements())
		{
			Castle cs = (Castle)e.nextElement();
			if (!cs.visited)
			{
				cs.died();
				castles.remove(cs.obj);
			}
		}
	}
	
	int xsize = 64, ysize = 72;
	int xrand, yrand;
		
	int dirs[] = { 8, 1, 2, 7, 0, 3, 6, 5, 4};
	// int dirs[] = { 1, 2, 3, 4, 5, 6, 7, 8, 0};

	public void obj_safe_squares(IObject o)
	{
		int i, j;
		for(i = 0; i < 9; i++)
			bool_arr[i] = true;

		for(i = 0; i < 9; i++)
		{
			int x = XCoord(o.getX(), dirs[i]);
			int y = YCoord(o.getY(), dirs[i]);
			if (x < 0 || x >= World.WIDTH)
			{
				bool_arr[i] = false;
				continue;
			}
			if (y < 0 || y >= World.HEIGHT)
			{
				bool_arr[i] = false;
				continue;
			}

			for(j = 0; j < 9; j++)
			{
				int cx = XCoord(x, dirs[j]);
				int cy = YCoord(y, dirs[j]);
				if (cx < 0 || cx >= World.WIDTH)
					continue;
				if (cy < 0 || cy >= World.HEIGHT)
					continue;

				IObject ob = World.getObjectAt(cx, cy);
				if (ob == null)
					continue;
				if (ob.getRuler() != this)
				{
					bool_arr[i] = false;
					break;
				}
			}
		}
	}

	void move_peasants()
	{
		int size = p.length;
		for(int i = 0; i < size; i++)
		{
			obj_safe_squares(p[i]);
			int id = p[i].getId();
			int xdelta = p[i].getX()-((xrand+id*234321)%xsize);
			int ydelta = p[i].getY()-((yrand+id*234321)%ysize);
			if( xdelta != 0 ) xdelta /= -Math.abs(xdelta);
			if( ydelta != 0 ) ydelta /= -Math.abs(ydelta);
			int dir = dirs[xdelta+3*ydelta+4];
			boolean moved = false;
			

			if (bool_arr[xdelta+3*ydelta+4])
			{
				if( dir == 0 ){
					dir = rand.nextInt(8)+1;
					xrand=rand.nextInt(23443857);
					yrand=rand.nextInt(23443857);
					}
				move(p[i],dir);
				moved = true;
			}
			
			if (!moved)
			{
			for(int j = 0; j < 8; j++)
			{
				int x = XCoord(p[i].getX(), dirs[j]);
				int y = YCoord(p[i].getY(), dirs[j]);
				if (x < 0 || x >= World.WIDTH)
					continue;
				if (y < 0 || y >= World.HEIGHT)
					continue;
				// if (null != World.getObjectAt(x, y))
				//	continue;
				if (bool_arr[j] && World.getLandOwner(x, y) != this)
				{
					move(p[i], dirs[j]);
					moved = true;
					break;
				}
			}
			}
			if (!moved)
				move(p[i],dir);
		}
	}
}

